@@ -47,6 +47,47 @@ def test_capability_with_forced_tty():
     child()
 
 
+def test_basic_url():
+    """force styling should return sequences even for non-ttys."""
+    @as_subprocess
+    def child():
+        # given
+        t = TestTerminal(stream=six.StringIO(), force_styling=True)
+        given_url = 'https://blessed.readthedocs.org'
+        given_text = 'documentation'
+        expected_output = ('\x1b]8;;{0}\x1b\\{1}\x1b]8;;\x1b\\'
+                           .format(given_url, given_text))
+
+        # exercise
+        result = t.link(given_url, 'documentation')
+
+        # verify
+        assert repr(result) == repr(expected_output)
+
+    child()
+
+
+def test_url_with_id():
+    """force styling should return sequences even for non-ttys."""
+    @as_subprocess
+    def child():
+        # given
+        t = TestTerminal(stream=six.StringIO(), force_styling=True)
+        given_url = 'https://blessed.readthedocs.org'
+        given_text = 'documentation'
+        given_url_id = '123'
+        expected_output = ('\x1b]8;id={0};{1}\x1b\\{2}\x1b]8;;\x1b\\'
+                           .format(given_url_id, given_url, given_text))
+
+        # exercise
+        result = t.link(given_url, 'documentation', given_url_id)
+
+        # verify
+        assert repr(result) == repr(expected_output)
+
+    child()
+
+
 def test_parametrization():
     """Test parameterizing a capability."""
     @as_subprocess
@@ -430,31 +471,27 @@ def test_nice_formatting_errors(all_terms):
         try:
             t.bold_misspelled('hey')
             assert not t.is_a_tty or False, 'Should have thrown exception'
-        except TypeError:
-            e = sys.exc_info()[1]
-            assert 'probably misspelled' in e.args[0]
+        except TypeError as e:
+            assert 'Unknown terminal capability,' in e.args[0]
         try:
             t.bold_misspelled(u'hey')  # unicode
             assert not t.is_a_tty or False, 'Should have thrown exception'
-        except TypeError:
-            e = sys.exc_info()[1]
-            assert 'probably misspelled' in e.args[0]
+        except TypeError as e:
+            assert 'Unknown terminal capability,' in e.args[0]
 
         try:
             t.bold_misspelled(None)  # an arbitrary non-string
             assert not t.is_a_tty or False, 'Should have thrown exception'
-        except TypeError:
-            e = sys.exc_info()[1]
-            assert 'probably misspelled' not in e.args[0]
+        except TypeError as e:
+            assert 'Unknown terminal capability,' not in e.args[0]
 
         if platform.python_implementation() != 'PyPy':
             # PyPy fails to toss an exception, Why?!
             try:
                 t.bold_misspelled('a', 'b')  # >1 string arg
                 assert not t.is_a_tty or False, 'Should have thrown exception'
-            except TypeError:
-                e = sys.exc_info()[1]
-                assert 'probably misspelled' in e.args[0], e.args
+            except TypeError as e:
+                assert 'Unknown terminal capability,' in e.args[0], e.args
 
     child(all_terms)
 
@@ -485,6 +522,7 @@ def test_padd():
         assert Sequence('xyz\b-', term).padd() == u'xy-'
         assert Sequence('xxxx\x1b[3Dzz', term).padd() == u'xzz'
         assert Sequence('\x1b[3D', term).padd() == u''  # "Trim left"
+        assert Sequence(term.red('xxxx\x1b[3Dzz'), term).padd() == term.red(u'xzz')
 
     kind = 'xterm-256color'
     if platform.system() == 'Windows':
@@ -512,3 +550,52 @@ def test_split_seqs(all_terms):
             assert result == expected
 
     child(all_terms)
+
+
+def test_formatting_other_string(all_terms):
+    """FormattingOtherString output depends on how it's called"""
+    @as_subprocess
+    def child(kind):
+        t = TestTerminal(stream=six.StringIO(), kind=kind, force_styling=True)
+
+        assert (t.move_left == t.cub1)
+        assert (t.move_left() == t.cub1)
+        assert (t.move_left(2) == t.cub(2))
+
+        assert (t.move_right == t.cuf1)
+        assert (t.move_right() == t.cuf1)
+        assert (t.move_right(2) == t.cuf(2))
+
+        assert (t.move_up == t.cuu1)
+        assert (t.move_up() == t.cuu1)
+        assert (t.move_up(2) == t.cuu(2))
+
+        assert (t.move_down == t.cud1)
+        assert (t.move_down() == t.cud1)
+        assert (t.move_down(2) == t.cud(2))
+
+    child(all_terms)
+
+
+def test_termcap_match_optional():
+    """When match_optional is given, numeric matches are optional"""
+    from blessed.sequences import Termcap
+    @as_subprocess
+    def child():
+        t = TestTerminal(force_styling=True)
+        cap = Termcap.build('move_right', t.cuf, 'cuf', nparams=1,
+                            match_grouped=True, match_optional=True)
+
+        # Digits absent
+        assert cap.re_compiled.match(t.cuf1).group(1) is None
+
+        # Digits present
+        assert cap.re_compiled.match(t.cuf()).group(1) == '0'
+        assert cap.re_compiled.match(t.cuf(1)).group(1) == '1'
+        assert cap.re_compiled.match(t.cuf(22)).group(1) == '22'
+
+        # Make sure match is not too generalized
+        assert cap.re_compiled.match(t.cub(2)) is None
+        assert cap.re_compiled.match(t.cub1) is None
+
+    child()
